@@ -1,6 +1,7 @@
 """
 Pytest fixtures and config.
 """
+
 import asyncio
 import email.header
 import email.message
@@ -15,6 +16,7 @@ from typing import Any, Callable, Coroutine, List, Optional, Tuple, Type, Union
 
 import hypothesis
 import pytest
+import pytest_asyncio
 import trustme
 from aiosmtpd.controller import Controller as SMTPDController
 from aiosmtpd.smtp import SMTP as SMTPD
@@ -22,6 +24,7 @@ from aiosmtpd.smtp import SMTP as SMTPD
 from aiosmtplib import SMTP, SMTPStatus
 
 from .auth import DummySMTPAuth
+from .compat import cleanup_server
 from .smtpd import RecordingHandler, TestSMTPD
 
 
@@ -49,10 +52,6 @@ class ParamFixtureRequest(pytest.FixtureRequest):
     param: Any
 
 
-class AsyncPytestWarning(pytest.PytestWarning):
-    pass
-
-
 class EchoServerProtocol(asyncio.Protocol):
     def connection_made(self, transport: asyncio.BaseTransport) -> None:
         self.transport = transport
@@ -98,6 +97,18 @@ def pytest_sessionfinish(session: pytest.Session, exitstatus: int) -> None:
         asyncio.set_event_loop_policy(original_event_loop_policy)
 
 
+@pytest_asyncio.fixture
+def debug_event_loop(
+    event_loop: asyncio.AbstractEventLoop,
+) -> asyncio.AbstractEventLoop:
+    previous_debug = event_loop.get_debug()
+    event_loop.set_debug(True)
+
+    yield event_loop
+
+    event_loop.set_debug(previous_debug)
+
+
 # Session scoped static values #
 
 
@@ -399,9 +410,9 @@ def smtpd_mock_response_done() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
 
 
 @pytest.fixture(scope="session")
-def smtpd_mock_response_done_then_close() -> Callable[
-    [SMTPD], Coroutine[Any, Any, None]
-]:
+def smtpd_mock_response_done_then_close() -> (
+    Callable[[SMTPD], Coroutine[Any, Any, None]]
+):
     async def mock_response_done_then_close(
         smtpd: SMTPD, *args: Any, **kwargs: Any
     ) -> None:
@@ -409,7 +420,7 @@ def smtpd_mock_response_done_then_close() -> Callable[
             smtpd.session.host_name = args[0]
         await smtpd.push("250 done")
         await smtpd.push("221 bye now")
-        await smtpd.transport.close()
+        smtpd.transport.close()
 
     return mock_response_done_then_close
 
@@ -423,14 +434,14 @@ def smtpd_mock_response_error() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
 
 
 @pytest.fixture(scope="session")
-def smtpd_mock_response_error_disconnect() -> Callable[
-    [SMTPD], Coroutine[Any, Any, None]
-]:
+def smtpd_mock_response_error_disconnect() -> (
+    Callable[[SMTPD], Coroutine[Any, Any, None]]
+):
     async def mock_response_error_disconnect(
         smtpd: SMTPD, *args: Any, **kwargs: Any
     ) -> None:
         await smtpd.push("501 error")
-        await smtpd.transport.close()
+        smtpd.transport.close()
 
     return mock_response_error_disconnect
 
@@ -438,7 +449,7 @@ def smtpd_mock_response_error_disconnect() -> Callable[
 @pytest.fixture(scope="session")
 def smtpd_mock_response_bad_data() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
     async def mock_response_bad_data(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        smtpd._writer.write(b"250 \xFF\xFF\xFF\xFF\r\n")
+        smtpd._writer.write(b"250 \xff\xff\xff\xff\r\n")
         await smtpd._writer.drain()
 
     return mock_response_bad_data
@@ -509,15 +520,15 @@ def smtpd_mock_response_unavailable() -> Callable[[SMTPD], Coroutine[Any, Any, N
         smtpd: SMTPD, *args: Any, **kwargs: Any
     ) -> None:
         await smtpd.push("421 retry in 5 minutes")
-        await smtpd.transport.close()
+        smtpd.transport.close()
 
     return mock_response_unavailable
 
 
 @pytest.fixture(scope="session")
-def smtpd_mock_response_tls_not_available() -> Callable[
-    [SMTPD], Coroutine[Any, Any, None]
-]:
+def smtpd_mock_response_tls_not_available() -> (
+    Callable[[SMTPD], Coroutine[Any, Any, None]]
+):
     async def mock_tls_not_available(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
         await smtpd.push("454 please login")
 
@@ -525,14 +536,14 @@ def smtpd_mock_response_tls_not_available() -> Callable[
 
 
 @pytest.fixture(scope="session")
-def smtpd_mock_response_tls_ready_disconnect() -> Callable[
-    [SMTPD], Coroutine[Any, Any, None]
-]:
+def smtpd_mock_response_tls_ready_disconnect() -> (
+    Callable[[SMTPD], Coroutine[Any, Any, None]]
+):
     async def mock_response_tls_ready_disconnect(
         smtpd: SMTPD, *args: Any, **kwargs: Any
     ) -> None:
         await smtpd.push("220 go for it")
-        await smtpd.transport.close()
+        smtpd.transport.close()
 
     return mock_response_tls_ready_disconnect
 
@@ -540,7 +551,7 @@ def smtpd_mock_response_tls_ready_disconnect() -> Callable[
 @pytest.fixture(scope="session")
 def smtpd_mock_response_disconnect() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
     async def mock_response_disconnect(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        await smtpd.transport.close()
+        smtpd.transport.close()
 
     return mock_response_disconnect
 
@@ -548,15 +559,15 @@ def smtpd_mock_response_disconnect() -> Callable[[SMTPD], Coroutine[Any, Any, No
 @pytest.fixture(scope="session")
 def smtpd_mock_response_eof() -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
     async def mock_response_eof(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
-        await smtpd.transport.write_eof()
+        smtpd.transport.write_eof()
 
     return mock_response_eof
 
 
 @pytest.fixture(scope="session")
-def smtpd_mock_response_error_with_code_factory() -> Callable[
-    [str], Callable[[SMTPD], Coroutine[Any, Any, None]]
-]:
+def smtpd_mock_response_error_with_code_factory() -> (
+    Callable[[str], Callable[[SMTPD], Coroutine[Any, Any, None]]]
+):
     def factory(error_code: str) -> Callable[[SMTPD], Coroutine[Any, Any, None]]:
         async def mock_error_response(smtpd: SMTPD, *args: Any, **kwargs: Any) -> None:
             await smtpd.push(f"{error_code} error")
@@ -632,7 +643,10 @@ def smtpd_server(
 
     def close_server() -> None:
         server.close()
-        event_loop.run_until_complete(server.wait_closed())
+        try:
+            event_loop.run_until_complete(cleanup_server(server))
+        except RuntimeError:
+            pass
 
     request.addfinalizer(close_server)
 
@@ -667,7 +681,10 @@ def smtpd_server_smtputf8(
 
     def close_server() -> None:
         server.close()
-        event_loop.run_until_complete(server.wait_closed())
+        try:
+            event_loop.run_until_complete(cleanup_server(server))
+        except RuntimeError:
+            pass
 
     request.addfinalizer(close_server)
 
@@ -688,9 +705,10 @@ def echo_server(
 
     def close_server() -> None:
         server.close()
-        event_loop.run_until_complete(server.wait_closed())
-
-    request.addfinalizer(close_server)
+        try:
+            event_loop.run_until_complete(cleanup_server(server))
+        except RuntimeError:
+            pass
 
     return server
 
@@ -723,7 +741,10 @@ def smtpd_server_socket_path(
 
     def close_server() -> None:
         server.close()
-        event_loop.run_until_complete(server.wait_closed())
+        try:
+            event_loop.run_until_complete(cleanup_server(server))
+        except RuntimeError:
+            pass
 
     request.addfinalizer(close_server)
 
@@ -759,7 +780,10 @@ def smtpd_server_tls(
 
     def close_server() -> None:
         server.close()
-        event_loop.run_until_complete(server.wait_closed())
+        try:
+            event_loop.run_until_complete(cleanup_server(server))
+        except RuntimeError:
+            pass
 
     request.addfinalizer(close_server)
 
