@@ -1,6 +1,7 @@
 """
 Connectivity tests.
 """
+
 import asyncio
 import pathlib
 import socket
@@ -104,8 +105,7 @@ async def test_close_on_connect_raises_connect_error(
     with pytest.raises(SMTPConnectError):
         await smtp_client.connect()
 
-    assert smtp_client.transport is None
-    assert smtp_client.protocol is None
+    assert not smtp_client.is_connected
 
 
 async def test_421_closes_connection(
@@ -150,8 +150,7 @@ async def test_disconnected_server_raises_on_client_read(
     with pytest.raises(SMTPServerDisconnected):
         await smtp_client.execute_command(b"NOOP")
 
-    assert smtp_client.protocol is None
-    assert smtp_client.transport is None
+    assert not smtp_client.is_connected
 
 
 async def test_disconnected_server_raises_on_client_write(
@@ -168,8 +167,7 @@ async def test_disconnected_server_raises_on_client_write(
     with pytest.raises(SMTPServerDisconnected):
         await smtp_client.execute_command(b"NOOP")
 
-    assert smtp_client.protocol is None
-    assert smtp_client.transport is None
+    assert not smtp_client.is_connected
 
 
 async def test_disconnected_server_raises_on_data_read(
@@ -189,8 +187,7 @@ async def test_disconnected_server_raises_on_data_read(
     with pytest.raises(SMTPServerDisconnected):
         await smtp_client.data("A MESSAGE")
 
-    assert smtp_client.protocol is None
-    assert smtp_client.transport is None
+    assert not smtp_client.is_connected
 
 
 async def test_disconnected_server_raises_on_data_write(
@@ -208,8 +205,7 @@ async def test_disconnected_server_raises_on_data_write(
     with pytest.raises(SMTPServerDisconnected):
         await smtp_client.data("A MESSAGE\nLINE2")
 
-    assert smtp_client.protocol is None
-    assert smtp_client.transport is None
+    assert not smtp_client.is_connected
 
 
 async def test_disconnected_server_raises_on_starttls(
@@ -227,13 +223,12 @@ async def test_disconnected_server_raises_on_starttls(
     async def mock_ehlo_or_helo_if_needed() -> None:
         pass
 
-    smtp_client._ehlo_or_helo_if_needed = mock_ehlo_or_helo_if_needed  # type: ignore
+    smtp_client._ehlo_or_helo_if_needed = mock_ehlo_or_helo_if_needed
 
     with pytest.raises(SMTPServerDisconnected):
         await smtp_client.starttls(timeout=1.0)
 
-    assert smtp_client.protocol is None
-    assert smtp_client.transport is None
+    assert not smtp_client.is_connected
 
 
 async def test_context_manager(
@@ -279,7 +274,7 @@ async def test_context_manager_exception_quits(
 ) -> None:
     with pytest.raises(ZeroDivisionError):
         async with smtp_client:
-            1 / 0
+            1 / 0  # noqa
 
     assert received_commands[-1][0] == "QUIT"
 
@@ -411,8 +406,8 @@ async def test_disconnected_server_get_transport_info(
     with pytest.raises(SMTPServerDisconnected):
         await smtp_client.execute_command(b"NOOP")
 
-    with pytest.raises(SMTPServerDisconnected):
-        await smtp_client.get_transport_info("sslcontext")
+    with pytest.raises(SMTPServerDisconnected, match="Server not connected"):
+        smtp_client.get_transport_info("sslcontext")
 
 
 async def test_disconnected_server_data(
@@ -432,7 +427,7 @@ async def test_disconnected_server_data(
     async def mock_ehlo_or_helo_if_needed() -> None:
         pass
 
-    smtp_client._ehlo_or_helo_if_needed = mock_ehlo_or_helo_if_needed  # type: ignore
+    smtp_client._ehlo_or_helo_if_needed = mock_ehlo_or_helo_if_needed
 
     with pytest.raises(SMTPServerDisconnected):
         await smtp_client.data("123")
