@@ -1,14 +1,18 @@
 """
 Protocol level tests.
 """
+
 import asyncio
+import gc
 import socket
 import ssl
 
 import pytest
 
 from aiosmtplib import SMTPResponseException, SMTPServerDisconnected
-from aiosmtplib.protocol import SMTPProtocol
+from aiosmtplib.protocol import FlowControlMixin, SMTPProtocol
+
+from .compat import cleanup_server
 
 
 pytestmark = pytest.mark.asyncio()
@@ -59,13 +63,13 @@ async def test_protocol_read_limit_overrun(
     monkeypatch.setattr("aiosmtplib.protocol.MAX_LINE_LENGTH", 128)
 
     with pytest.raises(SMTPResponseException) as exc_info:
-        await protocol.execute_command(b"TEST\n", timeout=1.0)
+        await protocol.execute_command(b"TEST\n", timeout=1.0)  # type: ignore
 
     assert exc_info.value.code == 500
     assert "Response too long" in exc_info.value.message
 
     server.close()
-    await server.wait_closed()
+    await cleanup_server(server)
 
 
 async def test_protocol_connected_check_on_read_response(
@@ -120,10 +124,10 @@ async def test_error_on_readline_with_partial_line(
     _, protocol = await asyncio.wait_for(connect_future, timeout=1.0)
 
     with pytest.raises(SMTPServerDisconnected):
-        await protocol.read_response(timeout=1.0)
+        await protocol.read_response(timeout=1.0)  # type: ignore
 
     server.close()
-    await server.wait_closed()
+    await cleanup_server(server)
 
 
 async def test_protocol_response_waiter_unset(
@@ -153,10 +157,10 @@ async def test_protocol_response_waiter_unset(
     monkeypatch.setattr(protocol, "_response_waiter", None)
 
     with pytest.raises(SMTPServerDisconnected):
-        await protocol.execute_command(b"TEST\n", timeout=1.0)
+        await protocol.execute_command(b"TEST\n", timeout=1.0)  # type: ignore
 
     server.close()
-    await server.wait_closed()
+    await cleanup_server(server)
 
 
 async def test_protocol_data_received_called_twice(
@@ -185,13 +189,13 @@ async def test_protocol_data_received_called_twice(
 
     _, protocol = await asyncio.wait_for(connect_future, timeout=1.0)
 
-    response = await protocol.execute_command(b"TEST\n", timeout=1.0)
+    response = await protocol.execute_command(b"TEST\n", timeout=1.0)  # type: ignore
 
     assert response.code == 220
     assert response.message == "Hi"
 
     server.close()
-    await server.wait_closed()
+    await cleanup_server(server)
 
 
 async def test_protocol_eof_response(
@@ -200,7 +204,7 @@ async def test_protocol_eof_response(
     async def client_connected(
         reader: asyncio.StreamReader, writer: asyncio.StreamWriter
     ) -> None:
-        writer.transport.abort()
+        writer.transport.abort()  # type: ignore
 
     server = await asyncio.start_server(
         client_connected, host=bind_address, port=0, family=socket.AF_INET
@@ -213,4 +217,64 @@ async def test_protocol_eof_response(
     await asyncio.wait_for(connect_future, timeout=1.0)
 
     server.close()
-    await server.wait_closed()
+    await cleanup_server(server)
+
+
+async def test_protocol_exception_cleanup_warning(
+    caplog: pytest.LogCaptureFixture,
+    debug_event_loop: asyncio.AbstractEventLoop,
+    bind_address: str,
+    hostname: str,
+) -> None:
+    async def client_connected(
+        reader: asyncio.StreamReader, writer: asyncio.StreamWriter
+    ) -> None:
+        await reader.read(1000)
+        writer.write(b"220 Hi\r\n")
+        await writer.drain()
+
+        await reader.read(1000)
+        writer.write(b"221 Bye\r\n")
+        await writer.drain()
+
+        writer.transport.close()
+
+    server = await asyncio.start_server(
+        client_connected, host=bind_address, port=0, family=socket.AF_INET
+    )
+    server_port = server.sockets[0].getsockname()[1] if server.sockets else 0
+
+    connect_future = debug_event_loop.create_connection(
+        SMTPProtocol, host=hostname, port=server_port
+    )
+    _, protocol = await asyncio.wait_for(connect_future, timeout=1.0)
+
+    await protocol.execute_command(b"HELO\n", timeout=1.0)
+    await protocol.execute_command(b"QUIT\n", timeout=1.0)
+
+    del protocol
+    # Force garbage collection
+    gc.collect()
+
+    server.close()
+    await cleanup_server(server)
+
+    assert "Future exception was never retrieved" not in caplog.text
+
+
+async def test_flow_control_drain(
+    event_loop: asyncio.AbstractEventLoop,
+):
+    # Adapted from stdlib
+    drained = 0
+
+    async def drainer(stream):
+        nonlocal drained
+        await stream._drain_helper()
+        drained += 1
+
+    stream = FlowControlMixin(event_loop)
+    stream.pause_writing()
+    event_loop.call_later(0.1, stream.resume_writing)
+    await asyncio.gather(*[drainer(stream) for _ in range(10)])
+    assert drained == 10
